/*
 * SecureServerToolkit.java
 */
package edu.stanford.peer.secrmi;

import java.io.*;
import java.rmi.RemoteException;
import java.rmi.server.RMIClientSocketFactory;
import java.security.*;
import java.security.cert.CertificateException;
import javax.net.ssl.SSLSocketFactory;

import com.sun.net.ssl.*;

/**
 * The SSLRMIClientSocketFactory class 
 * is Serializable, implements RMIClientSocketFactory, and produces SSL sockets
 *
 * @author  Ninghui Li
 */

public class SSLRMIClientSocketFactory 
        implements RMIClientSocketFactory, Serializable
{
	/* the trust store */
	transient private KeyStore   trustKS;

	/* the socket factory */
    transient private SSLSocketFactory  actualFactory;

	/**
	 * The classes KeyStore, SSLSocketFactory, and KeyManagerFactory are all
	 *  not Serializable, so we have to store the trust KeyStore in a 
	 *  TrustStore object 
     *
	 * @serial the trust store 
	 */
	private TrustStore ts;


    /**
	 * Create a SSLRMIClientSocketFactory, given a trust key store
	 * @param trustKS  the trust ket store
	 */
    public SSLRMIClientSocketFactory(KeyStore trustKS) 
	    throws NoSuchProviderException, NoSuchAlgorithmException, 
    		   KeyStoreException, CertificateException, UnrecoverableKeyException, 
    		   KeyManagementException
    {
		ts = new TrustStore(trustKS);
		this.trustKS = trustKS;
		initialize();
    }


    /**
	 * Create a SSL socket
	 * @param host  host name
	 * @param port  port number
	 */
    public java.net.Socket createSocket(String host, int port) throws IOException
	{
		if (actualFactory == null) {
			try {
				initialize();
			} catch (Exception e) {
				throw new RemoteException("SSL setup failed", e);
			}
		}
		return actualFactory.createSocket(host, port);
	}

	private void initialize() 
		throws NoSuchProviderException, NoSuchAlgorithmException, 
			   KeyStoreException, CertificateException, UnrecoverableKeyException, 
			   KeyManagementException
	{
		if (trustKS == null) 
		{
			trustKS = ts.toKeyStore("JKS");
		}
		
		TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509", "SunJSSE");
		tmf.init(trustKS);

		// Create SSL Context using default trust managers and default secure random
		com.sun.net.ssl.SSLContext sslContext = SSLContext.getInstance("TLS");
		sslContext.init(null, tmf.getTrustManagers(), null);

		actualFactory = sslContext.getSocketFactory();
	}

}