/*
 * SecureClientToolkit.java
 */


package edu.stanford.peer.secrmi;

import java.io.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.Naming;
import java.rmi.server.RemoteStub;
import java.rmi.server.UnicastRemoteObject;
import java.security.*;
import java.security.cert.CertificateException;

import com.sun.net.ssl.*;

/**
 * The SecureClientToolkit class 
 * 
 * is used by the client program to lookup a signed proxy from RMI registry
 * and to verify whether the downloaded proxy is correctly signed by the 
 * trusted public keys.  The class is instantiated with two trusted public 
 * keys: the trusted class key, i.e., the public key trusted for signing 
 * the class of the proxy, and the trusted objetc key, i.e., the public key 
 * trusted for signing the object of the proxy.  These two keys can be the 
 * same key or two different keys.
 *
 * @author  Ninghui Li
 */

public class SecureClientToolkit extends SecureRMIToolkit
{
	private PublicKey cvk;

	private PublicKey ovk;

    /**
     * Create a SecureClientToolkit object using the two trusted public keys.
     *
     * @param classKey the trusted class key
     * @param objectKey the trusted object key
     */
	public SecureClientToolkit(PublicKey classKey, PublicKey objectKey)
	{
		cvk = classKey;
		ovk = objectKey;
	}

    /**
     * Creates a SecureClientToolkit object using two KeyStore objects and
     * two aliases, which specify the two trusted public keys.
     *
     * @param ks1 the KeyStore that contains a certificate of the trusted class key
     * @param alias1 the alias of the certificate of the trusted class key in ks1
     * @param ks2 the KeyStore that contains a certificate of the trusted object key
     * @param alias2 the alias of the certificate of the trusted object key in ks2
     */
	public SecureClientToolkit(KeyStore classKS, String ckAlias,
	                           KeyStore objectKS, String okAlias) 
	     throws KeyStoreException
	{
		this(classKS.getCertificate(ckAlias).getPublicKey(),
		     objectKS.getCertificate(okAlias).getPublicKey());
	}

/*
	public SecureClientToolkit(String storefile, String pwd, String alias) 
	    throws KeyStoreException, FileNotFoundException,IOException,
               CertificateException, NoSuchAlgorithmException
    {
		KeyStore tks = loadKeyStore("JKS", storefile, pwd);
		//Get the certificate from the keystore
		servcert = tks.getCertificate(alias);
	}
*/

    /**
     * Looks up a signed proxy and verify that it is signed by the two trusted keys.
     * @param proxyAddress the address of the signed proxy
     * @returns the proxy
     */
	public Object lookupSigned(String proxyAddress) 
	    throws java.rmi.NotBoundException, VerificationFailureException, 
		       IOException, ClassNotFoundException
    {
		SignedRemoteObject sro = (SignedRemoteObject) Naming.lookup(proxyAddress);

		//Verify the signature on the object
		SignedObject so = sro.getSignedObject();

		try 
		{
			Signature verengine = Signature.getInstance(so.getAlgorithm());
			if(! so.verify(ovk, verengine)) 
			{
				throw new VerificationFailureException();
			}
		}
		catch (VerificationFailureException e) 
		{
			throw e;
		}
		catch (Exception e) 
		{
			throw new VerificationFailureException(e.toString());
		}
		return so.getObject();

		// Verify the signature on the SignedRemoteObject class here
		/*
		java.security.cert.Certificate[] certs =
		    sro.getClass().getProtectionDomain().getCodeSource().getCertificates();

		if((certs == null) || (certs.length == 0)) 
		{
			System.out.println("No Certificate on the class, unsafe to use it");
			return null;
		}

		try {
			certs[0].verify(servcert.getPublicKey());
		}
		catch(InvalidKeyException ex) 
		{
			//Key Was invalid or incorrect - Certificate not verified - return null
			System.out.println("Certificate on the class not verified correctly ");
			return null;
		}
		*/
	}

}
