/*
 * SecureRMIToolkit.java
 */

package edu.stanford.peer.secrmi;

import java.security.*;
import java.security.cert.CertificateException;
import java.io.*;

/**
 * The SecureRMIToolkit class 
 *
 * This class is the parent class of both SecureClientToolkit and 
 * SecureServerToolkit, providing static methods needed by both.
 * @author  Ninghui Li
 */

abstract class SecureRMIToolkit
{
    /**
	 * Load a key store from file.
     *
	 * @param type	   the type of the key store to load, e.g., JKS, jceks, etc.
	 * @param filename the name of the key store file
	 * @param icword   the word for checking the integrity of the key store; 
     *	               when it is null, no check is performed
	 */
	static final public KeyStore loadKeyStore
								 (String type, String filename, String icword)
            throws FileNotFoundException, IOException, KeyStoreException, 
				   CertificateException, NoSuchAlgorithmException
	{
		FileInputStream kss = new FileInputStream(filename);
		char[] ic = icword == null ? null : icword.toCharArray();
		return loadKeyStore(type, kss, ic);
	}
	
	/**
	 * Load a key store from an input stream.
	 *
	 * @param type	  the type of the key store to load, e.g., JKS, jceks, etc.
	 * @param in      the input stream of the keystore
	 * @param icchars the word for checking the integrity of the key store; 
	 *	               when it is null, no check is performed.
	 */
	static final public KeyStore loadKeyStore (String type, FileInputStream in, 
	                                           char[] icchars)
			throws IOException, KeyStoreException, 
				   CertificateException, NoSuchAlgorithmException
	{
		KeyStore ks = KeyStore.getInstance(type);
		ks.load(in, icchars);
		return ks;
	}

	/**
	 * Load a key store from file.
	 *
	 * @param type	   the type of the key store to load, e.g., JKS, jceks, etc.
	 * @param provider the name of the provider
	 * @param filename the name of the key store file
	 * @param icword   the word for checking the integrity of the key store; 
	 *	               when it is null, no check is performed
	 */
	static final public KeyStore loadKeyStore (String type, String provider, 
	                                           String filename, String icword) 
	        throws FileNotFoundException, IOException, KeyStoreException, 
				   CertificateException, NoSuchAlgorithmException,
				   NoSuchProviderException
	{
		FileInputStream kss = new FileInputStream(filename);
		char[] ic = icword == null ? null : icword.toCharArray();
		return loadKeyStore(type, provider, kss, ic);
	}
	
	/**
	 * Load a key store from an input stream.
	 *
	 * @param type	   the type of the key store to load, e.g., JKS, jceks, etc.
	 * @param provider the name of the provider
	 * @param in       the input stream of the keystore
	 * @param icchars  the word for checking the integrity of the key store; 
	 *	               when it is null, no check is performed
	 */
	static final public KeyStore loadKeyStore (String type, String provider, 
	                                     FileInputStream in, char[] icchars)
			throws IOException, KeyStoreException, NoSuchProviderException,
				   CertificateException, NoSuchAlgorithmException
	{
		KeyStore ks = KeyStore.getInstance(type, provider);
		ks.load(in, icchars);
		return ks;
	}

}