/*
 * SecureServerToolkit.java
 */

package edu.stanford.peer.secrmi;

import java.io.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.RemoteStub;
import java.rmi.server.UnicastRemoteObject;
import java.security.*;
import java.security.cert.CertificateException;

import com.sun.net.ssl.*;

/**
 * The SecureServerToolkit class 
 *
 * is used by a server program that provide secure and trusted RMI proxies.
 * It is used to produce signed RMI proxies that talk with RMI services 
 * through SSL.
 * The way to use this class is first to create a SecureServerToolkit object, 
 * then use it to export objects and register both the remote object and 
 * the signed proxy with a registry.
 * @author  Ninghui Li
 */
public class SecureServerToolkit extends SecureRMIToolkit
{
	/* The KeyStore the server needs to use */
	//private KeyStore  sks;

	/* The password needed to access the server KeyStore */
	//private char[]    sksPwd;

	/* The trust store the proxy is going to use */
	//private KeyStore  tks;

	/* The server socket factory used by services */
	private SSLRMIServerSocketFactory serviceSF;

	/* The client socket factory used by proxy */
	private SSLRMIClientSocketFactory proxySF;

	/* The key to sign proxies */
	private PrivateKey signingKey;

    /* The signature engine to sign proxies */
	private Signature  signingEngine;

	/**
	 * Creates a SecureServerToolkit object.
     *
	 * @param serviceKS the key store that holds the private key for services 
	 *                  to communicate act as SSL servers
	 * @param sksPwd    the password to access the private key in the key store
	 * @param proxyKS   the key store that holds certificates of that the proxy 
	 */
	public SecureServerToolkit(KeyStore serviceKS, String sksPwd, 
	                           KeyStore proxyKS)
		throws NoSuchAlgorithmException, KeyStoreException, NoSuchAlgorithmException,
			   NoSuchProviderException, UnrecoverableKeyException, CertificateException,
			   KeyManagementException, InvalidKeyException
	{
		serviceSF = new SSLRMIServerSocketFactory(serviceKS, sksPwd.toCharArray());
		proxySF = new SSLRMIClientSocketFactory(proxyKS);
	}

	/**
	 * Creates a SecureServerToolkit object.
     *
	 * @param serviceKS the key store services use to communicate 
	 *                  with proxies through SSL
	 * @param sksPwd    the password to access the service key store
	 * @param proxyKS   the key store that the proxy uses as a trust store
	 */
	public SecureServerToolkit(KeyStore serviceKS, String sksPwd,
							   KeyStore proxyKS, String signingKeyAlias, 
							   String signingAlg)
		throws NoSuchAlgorithmException, KeyStoreException, NoSuchAlgorithmException,
			   NoSuchProviderException, UnrecoverableKeyException, CertificateException,
				KeyManagementException, InvalidKeyException
	{
		this(serviceKS, sksPwd, proxyKS);
		setSignature((PrivateKey)serviceKS.getKey(signingKeyAlias, sksPwd.toCharArray()),
			         signingAlg);
	}

	/**
	 * Creates a SecureServerToolkit object.
     *
	 * @param serviceKS the key store services use to communicate 
	 *                  with proxies through SSL
	 * @param sksPwd    the password to access the service key store
	 * @param proxyKS   the key store that the proxy uses as a trust store
	 * @param signingAlg name of the algorithm to sign proxies
	 * @param provider   name of the security provider to sign proxies
	 */
	public SecureServerToolkit(KeyStore serviceKS, String sksPwd,
							   KeyStore proxyKS, String signingKeyAlias, 
							   String signingAlg, String provider)
		throws NoSuchAlgorithmException, KeyStoreException, NoSuchAlgorithmException,
				NoSuchProviderException, UnrecoverableKeyException, CertificateException,
				KeyManagementException, InvalidKeyException
	{
		this(serviceKS, sksPwd, proxyKS);
		setSignature((PrivateKey)serviceKS.getKey(signingKeyAlias, sksPwd.toCharArray()),
            		  signingAlg, provider);
	}


	synchronized public void setSignature(PrivateKey signingKey, String signingAlg)
			throws InvalidKeyException, NoSuchAlgorithmException
	{
		if (signingKey == null) 
		{
			throw new InvalidKeyException("Empty key");
		}
			
		this.signingKey = signingKey;
		this.signingEngine = Signature.getInstance(signingAlg);
	}

	synchronized public void setSignature(PrivateKey signingKey, String signingAlg, String provider)
			throws NoSuchAlgorithmException, NoSuchProviderException, InvalidKeyException
	{
		if (signingKey == null) 
		{
			throw new InvalidKeyException("Empty key");
		}
	
		this.signingKey = signingKey;
		this.signingEngine = Signature.getInstance(signingAlg, provider);
	}

    /**
     * Exports a remote object so that proxies of the remote object communicate
     * with the object through SSL and creates a signed proxy.
     * @param object the object to be exported
     * @returns the signed proxy
     */
	final public SignedRemoteObject exportSignedObject(Remote object)
			throws RemoteException, IOException, InvalidKeyException, 
			SignatureException
	{
		Remote proxy = exportObject(object);

		return new SignedRemoteObject((Serializable)proxy, 
		                              signingKey, signingEngine);
	}

    /**
     * Exports a remote object so that proxies of the remote object communicate
     * with the object through SSL.
     * @param object the object to be exported
     * @returns the result of exporting the object
     */
	final public Remote exportObject(Remote object)
			throws RemoteException
	{
		return UnicastRemoteObject.exportObject(object, 0, proxySF, serviceSF); 
	}


	/*
	 * An SSL client needs a trust store to store the trusted root public
	 * key.  An SSL server needs a key store for private key and certificates.
	 * They should be different keys.
	 */
}