/*
 * TrustStore.java
 */

package edu.stanford.peer.secrmi;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Enumeration;
import java.util.Iterator;
import java.security.cert.Certificate;
import java.security.KeyStore;
import java.security.KeyStoreException;

/*
 Note that there are only two kinds of entries in a KeyStore, either a 
 certificate entry, which contains a self-signed root certificate, or a
 key entry (a private key entry or a secret key entry).  
 A private key entry contains a private key and a chain of certificates.  
 A public key cannot be accompanied by a chain of certificates.
 Essentially, the KeyStore should be extended.
 */

/**
 * The TrustStore class is essentially a Serializable KeyStore.  
 * It is needed to maintain status information in SSLRMIClientSocketFactory. 
 * Because it is Serializable, it can contain only public information,
 * i.e., certificate entries, but not keys.
 *
 * @version 1.01 01/14/02
 * @author  Ninghui Li
 */
public class TrustStore implements Serializable
{
	/**
	 * @serial maps alias to certificates
	 */
	private HashMap  entries = new HashMap();

    /**
	 * Creates a TrustStore object using a KeyStore object.
	 * @param ks the key store to be converted to a trust store
	 */
	public TrustStore(KeyStore ks)
		throws KeyStoreException
	{
		Enumeration aliases = ks.aliases();
		while (aliases.hasMoreElements()) 
		{
			String alias = (String) aliases.nextElement();
			if (ks.isCertificateEntry(alias)) 
			{
				entries.put(alias, ks.getCertificate(alias));
			}
		}
	}

	/**
	 * Translate itself to a KeyStore object of the given type.
	 * @param type the type of the key store wanted, e.g., JKS
	 * @return the resulting key store
	 */
	public KeyStore toKeyStore(String type)
		throws KeyStoreException
	{
		KeyStore ks = KeyStore.getInstance(type);
		try 
		{
			// Initialize an empty KeyStore
			ks.load(null, null);
		}
		catch (Exception e) 
		{
			// this shouldn't occur, since we init it with nothing
		}
		
		Iterator it = entries.keySet().iterator();
		while (it.hasNext()) 
		{
			String alias = (String) it.next();
			ks.setCertificateEntry(alias, (Certificate)entries.get(alias));
		}
		return ks;
	}
}